@@ -36,11 +36,12 @@ Finally, ack takes settings from the command line.
 use strict;
 use warnings;
 
+use App::Ack ();
 use App::Ack::ConfigDefault;
 use Cwd 3.00 ();
 use File::Spec 3.00;
 
-use if ($^O =~ /MSWin32/ ? 1 : 0), "Win32";
+use if ($^O eq 'MSWin32'), 'Win32';
 
 =head1 METHODS
 
@@ -50,25 +51,20 @@ Creates a new config finder.
 
 =cut
 
-our $is_win = 0;
-
 sub new {
     my ( $class ) = @_;
 
-    $is_win = $^O =~ /MSWin32/,
-
     return bless {}, $class;
 }
 
 sub _remove_redundancies {
     my ( @configs ) = @_;
 
-    if ( $is_win ) {
-        # inode stat always returns 0 on windows,
-        # so just check filenames
+    if ( $App::Ack::is_windows ) {
+        # inode stat always returns 0 on windows, so just check filenames.
         my (%seen, @uniq);
 
-        foreach my $path (@configs) {
+        foreach my $path (map { $_->{path} } @configs) {
             push @uniq, $path unless $seen{$path};
             $seen{$path} = 1;
         }
@@ -80,12 +76,13 @@ sub _remove_redundancies {
 
         my %dev_and_inode_seen;
 
-        foreach my $path ( @configs ) {
+        foreach my $config ( @configs ) {
+            my $path = $config->{path};
             my ( $dev, $inode ) = (stat $path)[0, 1];
 
             if( defined($dev) ) {
                 if( $dev_and_inode_seen{"$dev:$inode"} ) {
-                    undef $path;
+                    undef $config;
                 }
                 else {
                     $dev_and_inode_seen{"$dev:$inode"} = 1;
@@ -114,37 +111,39 @@ sub _check_for_ackrc {
 
 =head2 $finder->find_config_files
 
-Locates config files, and returns
-a list of them.
+Locates config files, and returns a list of them.
 
 =cut
 
 sub find_config_files {
     my @config_files;
 
-    if( $is_win ) {
-        push @config_files, map { File::Spec->catfile($_, 'ackrc') } (
+    if ( $App::Ack::is_windows ) {
+        push @config_files, map { +{ path => File::Spec->catfile($_, 'ackrc') } } (
             Win32::GetFolderPath(Win32::CSIDL_COMMON_APPDATA()),
             Win32::GetFolderPath(Win32::CSIDL_APPDATA()),
         );
     }
     else {
-        push @config_files, '/etc/ackrc';
+        push @config_files, { path => '/etc/ackrc' };
     }
 
 
     if ( $ENV{'ACKRC'} && -f $ENV{'ACKRC'} ) {
-        push @config_files, $ENV{'ACKRC'};
+        push @config_files, { path => $ENV{'ACKRC'} };
     }
     else {
-        push @config_files, _check_for_ackrc($ENV{'HOME'});
+        push @config_files, map { +{ path => $_ } } _check_for_ackrc($ENV{'HOME'});
     }
 
-    my @dirs = File::Spec->splitdir(Cwd::getcwd());
+    my $cwd = Cwd::getcwd();
+    $cwd =~ /(.+)/;
+    $cwd = $1;
+    my @dirs = File::Spec->splitdir( $cwd );
     while(@dirs) {
         my $ackrc = _check_for_ackrc(@dirs);
         if(defined $ackrc) {
-            push @config_files, $ackrc;
+            push @config_files, { project => 1, path => $ackrc };
             last;
         }
         pop @dirs;
